# -*- coding: utf-8 -*-

"""Script for starting the pipeline and saving the results."""

from dataclasses import dataclass
from typing import Dict, Mapping, Optional

import pykeen
from pykeen.utilities.pipeline import Pipeline


@dataclass
class Results:
    """Results from PyKEEN."""

    config: Mapping
    pipeline: Pipeline
    results: Mapping


def run(config: Dict,
        output_directory: Optional[str] = None) -> Results:
    """Run PyKEEN using a given configuration."""
    results = pykeen.run(
        config=config,
        output_directory=output_directory,
    )

    return results



# -*- coding: utf-8 -*-

"""BioKEEN's command line interface helper."""

import click


def print_welcome_message():
    """Print welcome message."""
    click.secho('#################################################')
    click.secho("#\t\tWelcome to " + click.style("BioKEEN", bold=True) + "\t\t#")
    click.secho('#################################################')


def print_intro():
    """Print intro."""
    click.secho("This interface will assist you to configure your experiment.")
    click.secho("")
    click.secho(
        "BioKEEN can be run in two modes: \n"
        "1.) Training mode: BioKEEN trains a model based on a set of user-defined hyper-parameters.\n"
        "2.) Hyper-parameter optimization mode: "
        "Apply Random Search to determine the most appropriate set of hyper-parameter values"
    )



# -*- coding: utf-8 -*-

"""Conversion base classes."""

from abc import ABC, abstractmethod
from typing import Dict, Tuple

from pybel.constants import (
    ACTIVITY, ASSOCIATION, CORRELATIVE_RELATIONS, DECREASES, DIRECTLY_DECREASES, DIRECTLY_INCREASES, EQUIVALENT_TO,
    HAS_COMPONENT, INCREASES, IS_A, MODIFIER, OBJECT, PART_OF, REGULATES, RELATION,
)
from pybel.dsl import Abundance, BaseEntity, BiologicalProcess, MicroRna, NamedComplexAbundance, Pathology, Protein, Rna


class Converter(ABC):
    """A condition and converter for a BEL edge."""

    @staticmethod
    @abstractmethod
    def predicate(u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> bool:
        """Test a BEL edge."""

    @staticmethod
    @abstractmethod
    def convert(u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> Tuple[str, str, str]:
        """Convert a BEL edge."""


class SimpleConverter(Converter):
    """A class for converting the source and target that have simple names."""

    @classmethod
    def convert(cls, u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict):
        """Convert a BEL edge."""
        return (
            f'{u.namespace}:{u.identifier or u.name}',
            edge_data[RELATION],
            f'{v.namespace}:{v.identifier or v.name}',
        )


class TypedConverter(Converter):
    """A class for converting the source and target but replaces the relation."""

    target_relation = None

    @classmethod
    def convert(cls, u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict):
        """Convert a BEL edge."""
        return (
            f'{u.namespace}:{u.identifier or u.name}',
            cls.target_relation,
            f'{v.namespace}:{v.identifier or v.name}',
        )


class SimpleTypedPredicate(Converter):
    """Finds BEL statements like ``A(X) B C(Y)`` where relation B and types A and C are defined in the class."""

    subject_type = ...
    relation = ...
    object_type = ...

    @classmethod
    def predicate(cls, u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict):
        """Test a BEL edge."""
        return (
            isinstance(u, cls.subject_type) and
            edge_data[RELATION] == cls.relation and
            isinstance(v, cls.object_type)
        )


class _ConvertOnRelation(Converter):
    relation = ...

    @classmethod
    def predicate(cls, u, v, key, edge_data):
        """Test a BEL edge has a given relation."""
        return edge_data[RELATION] == cls.relation


class _PartOfConverter(SimpleTypedPredicate, TypedConverter):
    relation = PART_OF
    target_relation = 'partOf'


class PartOfNamedComplexConverter(_PartOfConverter):
    """Converts BEL statements like ``p(X) partOf complex(Y)``."""

    subject_type = Protein
    object_type = NamedComplexAbundance


class PartOfBiologicalProcess(_PartOfConverter):
    """Converts BEL statements like ``bp(X) partOf bp(Y)``."""

    subject_type = BiologicalProcess
    object_type = BiologicalProcess


class NamedComplexHasComponentConverter(SimpleTypedPredicate):
    """Converts BEL statements like ``complex(X) hasComponent p(Y)``."""

    subject_type = NamedComplexAbundance
    relation = HAS_COMPONENT
    object_type = Protein
    target_relation = 'partOf'

    @classmethod
    def convert(cls, u: BaseEntity, v: BaseEntity, key: str, data: Dict):
        """Convert a BEL edge."""
        return (
            f'{v.namespace}:{v.identifier or v.name}',
            cls.target_relation,
            f'{u.namespace}:{u.identifier or u.name}',
        )


'''
class TranslationConverter(TypedConverter):
    relation = TRANSLATED_TO
    target_relation = 'ribosomallyTranslatesTo'


class TranscriptionConverter(TypedConverter):
    relation = TRANSCRIBED_TO
    target_relation = 'ribosomallyTranslatesTo'
'''


class IsAConverter(_ConvertOnRelation):
    """Converts BEL statements like ``X isA Y``."""

    relation = IS_A
    target_relation = 'isA'


class EquivalenceConverter(_ConvertOnRelation):
    """Converts BEL statements like ``X eq Y``."""

    relation = EQUIVALENT_TO
    target_relation = 'equivalentTo'


class CorrelationConverter(SimpleConverter):
    """Converts BEL statements like ``A(B) pos|neg|noCorrelation C(D)``."""

    @staticmethod
    def predicate(u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> bool:
        """Test a BEL edge."""
        return edge_data[RELATION] in CORRELATIVE_RELATIONS


class AssociationConverter(Converter):
    """Converts BEL statements like ``a(X) -- path(Y)``."""

    @staticmethod
    def predicate(u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> bool:
        """Test a BEL edge."""
        return edge_data[RELATION] == ASSOCIATION

    @staticmethod
    def convert(u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> Tuple[str, str, str]:
        """Convert a BEL edge."""
        return (
            f'{u.namespace}:{u.identifier or u.name}',
            edge_data.get('association_type', ASSOCIATION),  # allow more specific association to be defined
            f'{v.namespace}:{v.identifier or v.name}',
        )


class DrugEffectConverter(SimpleConverter, SimpleTypedPredicate):
    """Converts BEL statements like ``a(X) ? path(Y)``."""

    subject_type = Abundance
    relation = ...
    object_type = Pathology


class DrugIndicationConverter(DrugEffectConverter):
    """Converts BEL statements like ``a(X) -| path(Y)``."""

    relation = DECREASES


class DrugSideEffectConverter(DrugEffectConverter):
    """Converts BEL statements like ``a(X) -> path(Y)``."""

    relation = INCREASES


class RegulatesAmountConverter(TypedConverter):
    """Converts BEL statements like ``A(B) reg C(D)``."""

    relation = REGULATES
    target_relation = 'regulatesAmountOf'

    @classmethod
    def predicate(cls, u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> bool:
        """Test a BEL edge."""
        object_modifier = edge_data.get(OBJECT)
        return edge_data[RELATION] == cls.relation and (not object_modifier or not object_modifier.get(MODIFIER))


class IncreasesAmountConverter(RegulatesAmountConverter):
    """Converts BEL statements like ``A(B) -> C(D)``."""

    relation = INCREASES
    target_relation = 'increasesAmountOf'


class DecreasesAmountConverter(RegulatesAmountConverter):
    """Converts BEL statements like ``A(B) -| C(D)``."""

    relation = DECREASES
    target_relation = 'decreasesAmountOf'


class RegulatesActivityConverter(TypedConverter):
    """Converts BEL statements like ``A(B) reg act(C(D) [, ma(E)])``."""

    relation = REGULATES
    target_relation = 'activityDirectlyRegulatesActivityOf'

    @classmethod
    def predicate(cls, u: BaseEntity, v: BaseEntity, key: str, edge_data: Dict) -> bool:
        """Test a BEL edge."""
        object_modifier = edge_data.get(OBJECT)
        return edge_data[RELATION] == cls.relation and object_modifier and object_modifier.get(MODIFIER) == ACTIVITY


class IncreasesActivityConverter(RegulatesActivityConverter):
    """Converts BEL statements like ``A(B) -> act(C(D) [, ma(E)])``."""

    relation = INCREASES
    target_relation = 'activityDirectlyPositivelyRegulatesActivityOf'


class DecreasesActivityConverter(RegulatesActivityConverter):
    """Converts BEL statements like ``A(B) -| act(C(D) [, ma(E)])``."""

    relation = DECREASES
    target_relation = 'activityDirectlyNegativelyRegulatesActivityOf'


class MiRNARegulatesExpressionConverter(TypedConverter, SimpleTypedPredicate):
    """Converts BEL statements like ``m(X) reg r(Y)``."""

    subject_type = MicroRna
    relation = REGULATES
    object_type = Rna
    target_relation = 'regulatesExpressionOf'


class MiRNAIncreasesExpressionConverter(MiRNARegulatesExpressionConverter):
    """Converts BEL statements like ``m(X) -> r(Y)``."""

    relation = INCREASES
    target_relation = 'increasesExpressionOf'


class MiRNADirectlyIncreasesExpressionConverter(MiRNARegulatesExpressionConverter):
    """Converts BEL statements like ``m(X) => r(Y)``."""

    relation = DIRECTLY_INCREASES
    target_relation = 'increasesExpressionOf'


class MiRNADecreasesExpressionConverter(MiRNARegulatesExpressionConverter):
    """Converts BEL statements like ``m(X) -| r(Y)``."""

    relation = DECREASES
    target_relation = 'repressesExpressionOf'


class MiRNADirectlyDecreasesExpressionConverter(MiRNARegulatesExpressionConverter):
    """Converts BEL statements like ``m(X) =| r(Y)``."""

    relation = DIRECTLY_DECREASES
    target_relation = 'repressesExpressionOf'



# -*- coding: utf-8 -*-

"""A command line interface for BioKEEN."""

import json
import logging
from typing import List, Optional, TextIO

import click

import pykeen
from bio2bel.constants import get_global_connection
from biokeen.build import iterate_source_paths
from biokeen.cli_utils.bio_2_bel_utils import install_bio2bel_module
from biokeen.cli_utils.prompt_utils import prompt_config
from pykeen.predict import start_predictions_pipeline

connection_option = click.option(
    '-c',
    '--connection',
    default=get_global_connection(),
    show_default=True,
    help='Bio2BEL database connection string',
)


@click.group()
@click.version_option()
def main():  # noqa: D401
    """A command line interface for BioKEEN."""


@main.command()
@connection_option
@click.option('-f', '--config', type=click.File())
@click.option('-r', '--rebuild', is_flag=True)
def start(config: Optional[TextIO], connection: str, rebuild: bool):
    """Start BioKEEN pipeline."""
    if config is None:
        config = prompt_config(connection, rebuild)
    else:
        config = json.load(config)

    pykeen.run(config)


@main.command()
@click.option('-m', '--model-directory', type=click.Path(file_okay=False, dir_okay=True))
@click.option('-d', '--data-directory', type=click.Path(file_okay=False, dir_okay=True))
def predict(model_directory: str, data_directory: str):
    """Use a trained model to make predictions."""
    start_predictions_pipeline(model_directory, data_directory)


@main.group()
def data():
    """Commands for data acquisition."""


@data.command()
def ls():
    """List built data."""
    for path in iterate_source_paths():
        click.echo(path)


@data.command()
@click.argument('names', nargs=-1)
@connection_option
@click.option('-r', '--rebuild', is_flag=True)
@click.option('-v', '--verbose', is_flag=True)
def get(names: List[str], connection: str, rebuild: bool, verbose: bool):
    """Install, populate, and build Bio2BEL repository."""
    if verbose:
        logging.basicConfig(level=logging.INFO)

    for name in names:
        install_bio2bel_module(name, connection, rebuild)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""CLI utils."""

from collections import OrderedDict
from typing import Dict

import click

from biokeen.cli_utils.bio_2_bel_utils import install_bio2bel_module
from biokeen.cli_utils.cli_print_msg_helper import print_intro, print_welcome_message
from biokeen.cli_utils.cli_query_helper import select_database
from pykeen.cli.prompt import (
    prompt_device, prompt_embedding_model, prompt_evaluation_parameters, prompt_execution_parameters,
    prompt_output_directory, prompt_random_seed, prompt_training_file,
)
from pykeen.cli.utils.cli_print_msg_helper import print_execution_mode_message, print_section_divider
from pykeen.constants import EXECUTION_MODE, HPO_MODE, TRAINING_MODE, TRAINING_SET_PATH

__all__ = [
    'prompt_config',
]


def prompt_config(connection: str, rebuild: bool) -> Dict:
    """Configure experiments."""
    config = OrderedDict()

    # Step 1: Welcome + Intro
    print_welcome_message()
    print_section_divider()
    print_intro()
    print_section_divider()

    # Step 2: Ask for data source
    is_biokeen_data_required = click.confirm('Do you want to use one of the databases provided by BioKEEN?',
                                             default=True)
    print_section_divider()

    if is_biokeen_data_required:
        database_name = select_database()
        config[TRAINING_SET_PATH] = install_bio2bel_module(name=database_name, connection=connection, rebuild=rebuild)
    else:
        prompt_training_file(config)

    print_section_divider()

    # Step 3: Ask for execution mode
    print_execution_mode_message()
    config[EXECUTION_MODE] = (
        TRAINING_MODE
        if click.confirm('Do you have hyper-parameters? If not, will begin hyper-parameter search.', default=False) else
        HPO_MODE
    )
    print_section_divider()

    # Step 4: Ask for model
    model_name = prompt_embedding_model()
    print_section_divider()

    # Step 5: Query parameters depending on the selected execution mode
    prompt_execution_parameters(config=config, model_name=model_name)
    print_section_divider()

    prompt_evaluation_parameters(config)

    print_section_divider()

    # Step 6: Please select a random seed
    prompt_random_seed(config)
    print_section_divider()

    # Step 7: Query device to train on
    prompt_device(config)
    print_section_divider()

    # Step 8: Define output directory
    prompt_output_directory(config)
    print_section_divider()

    return config



# -*- coding: utf-8 -*-

"""Utilities for BioKEEN."""

import importlib
import os
import sys
from typing import Optional

import click
import pkg_resources

from bio2bel import AbstractManager
from bio2bel.manager.bel_manager import BELManagerMixin
from biokeen.constants import DATA_DIR, EMOJI
from biokeen.convert import to_pykeen_file
from pybel import from_json_path, to_json_path


def _import_bio2bel_module(package: str):
    """Import a package, or install it."""
    try:
        b_module = importlib.import_module(package)

    except ImportError:
        click.secho(f'{EMOJI} pip install {package}', bold=True)
        # Install this package using pip
        # https://stackoverflow.com/questions/12332975/installing-python-module-within-code
        from pip._internal import main as pip_main
        r = pip_main(['install', package])
        if r != 0:  # command failed
            click.secho(f'{EMOJI} could not find {package} on PyPI. Try installing from GitHub with:', bold=True)
            name = package.split("_")[-1]
            click.echo(f'\n   pip install git+https://github.com/bio2bel/{name}.git\n')
            sys.exit(1)

        try:
            return importlib.import_module(package)
        except ImportError:
            click.secho(f'{EMOJI} failed to import {package}', bold=True)
            sys.exit(1)

    return b_module


def install_bio2bel_module(name: str, connection: str, rebuild: bool) -> Optional[str]:
    """Install Bio2BEL module."""
    if name == 'compath':  # special case for compath
        module_name = 'compath_resources'
    else:
        module_name = f'bio2bel_{name}'

    pykeen_df_path = os.path.join(DATA_DIR, f'{name}.keen.tsv')
    json_path = os.path.join(DATA_DIR, f'{name}.bel.json')

    if os.path.exists(pykeen_df_path) and not rebuild:
        click.secho(f'{EMOJI} {module_name} has already been retrieved. See: {pykeen_df_path}', bold=True)
        return pykeen_df_path

    if os.path.exists(json_path) and not rebuild:
        click.secho(f'{EMOJI} loaded {module_name} JSON: {json_path}', bold=True)
        graph = from_json_path(json_path)
        to_pykeen_file(graph, pykeen_df_path)
        return pykeen_df_path

    bio2bel_module = _import_bio2bel_module(module_name)
    click.secho(f'{EMOJI} imported {module_name}', bold=True)

    manager_cls = bio2bel_module.Manager

    if not issubclass(manager_cls, BELManagerMixin):
        version = pkg_resources.get_distribution(module_name).version
        click.secho(f'{EMOJI} {module_name} v{version} does not produce BEL', bold=True, fg='red')
        sys.exit(1)

    manager = manager_cls(connection=connection)

    if issubclass(manager_cls, AbstractManager):
        if not manager.is_populated():
            click.secho(f'{EMOJI} populating {module_name}', bold=True)
            manager.populate()
        else:
            click.secho(f'{EMOJI} {module_name} has already been populated', bold=True)

    click.secho(f'{EMOJI} generating BEL for {module_name}', bold=True)
    graph = manager.to_bel()

    click.echo(f'Summary: {graph.number_of_nodes()} nodes / {graph.number_of_edges()} edges')
    to_json_path(graph, json_path, indent=2)

    click.secho(f'{EMOJI} generating PyKEEN TSV for {module_name}', bold=True)
    success = to_pykeen_file(graph, pykeen_df_path)

    if success:
        click.secho(f'{EMOJI} wrote PyKEEN TSV to {pykeen_df_path}', bold=True)
        return pykeen_df_path

    click.secho(f'{EMOJI} no statements generated', bold=True, fg='red')



# -*- coding: utf-8 -*-

"""Simple tests for BioKEEN."""

import unittest

from biokeen.constants import VERSION


class TestImport(unittest.TestCase):
    """Simple tests for importing BioKEEN."""

    def test_version_type(self):
        """Test the type of the version string."""
        self.assertIsInstance(VERSION, str)



# -*- coding: utf-8 -*-

"""Constants for BioKEEN."""

import os

import easy_config

HERE = os.path.abspath(os.path.dirname(__file__))


class BiokeenConfig(easy_config.EasyConfig):
    """Configuration for BioKEEN."""

    NAME = 'biokeen'
    FILES = [
        os.path.join(os.path.expanduser('~'), '.config', 'biokeen.cfg'),
        os.path.join(os.path.expanduser('~'), '.config', 'config.ini'),
    ]

    #: the data directory where TSVs get exported
    data: str = os.path.abspath(os.path.join(HERE, os.pardir, os.pardir, 'data'))


biokeen_config = BiokeenConfig.load()

DATA_DIR = biokeen_config.data
os.makedirs(DATA_DIR, exist_ok=True)


VERSION = '0.0.11'
CONFIG_PATH = os.path.join(DATA_DIR, "configuration.json")

EMOJI = '🍩'

# Available databases
COMPATH_NAME = 'compath'
HIPPIE_NAME = 'hippie'
KEGG_NAME = 'kegg'
MIRTARBASE_NAME = 'mirtarbase'
MSIG_NAME = 'msig'
REACTOME_NAME = 'reactome'
SIDER_NAME = 'sider'
WIKIPATHWAYS_NAME = 'wikipathways'
DRUGBANK_NAME = 'drugbank'
ADEPTUS_NAME = 'adeptus'
HSDN_NAME = 'hsdn'
INTERPRO_NAME = 'interpro'

# ToDo: Add databases
DATABASES = [
    COMPATH_NAME,
    HIPPIE_NAME,
    KEGG_NAME,
    MIRTARBASE_NAME,
    MSIG_NAME,
    REACTOME_NAME,
    SIDER_NAME,
    WIKIPATHWAYS_NAME,
    DRUGBANK_NAME,
    ADEPTUS_NAME,
    HSDN_NAME,
    INTERPRO_NAME,
]

ID_TO_DATABASE_MAPPING = dict(enumerate(DATABASES, start=1))



# -*- coding: utf-8 -*-

"""Input and output for BEL conversion."""

import logging
from pathlib import Path
from typing import Optional, TextIO, Tuple, Union

import pandas as pd
from tqdm import tqdm

from pybel import BELGraph
from pybel.dsl import BaseEntity
from .converters import (
    AssociationConverter, CorrelationConverter, DecreasesAmountConverter, DrugIndicationConverter,
    DrugSideEffectConverter, EquivalenceConverter, IncreasesAmountConverter, IsAConverter,
    MiRNADecreasesExpressionConverter, MiRNADirectlyDecreasesExpressionConverter, NamedComplexHasComponentConverter,
    PartOfNamedComplexConverter, RegulatesActivityConverter, RegulatesAmountConverter, PartOfBiologicalProcess
)

__all__ = [
    'to_pykeen_file',
    'to_pykeen_df',
    'get_triple',
]

logger = logging.getLogger(__name__)


def to_pykeen_file(graph: BELGraph, file: Union[str, Path, TextIO]) -> bool:
    """Write the relationships in the BEL graph to a KEEN TSV file."""
    df = to_pykeen_df(graph)

    if len(df.index) == 0:
        return False

    df.to_csv(file, sep='\t', index=None, header=None)
    return True


def to_pykeen_df(graph: BELGraph) -> pd.DataFrame:
    """Get a pandas DataFrame representing the triples."""
    triples = (
        get_triple(graph, u, v, key)
        for u, v, key in tqdm(graph.edges(keys=True), total=graph.number_of_edges(), desc='preparing TSV')
    )

    # clean duplicates and Nones
    triples = list(sorted({triple for triple in triples if triple is not None}))

    return pd.DataFrame(triples, columns=['subject', 'predicate', 'object'])


def get_triple(graph: BELGraph, u: BaseEntity, v: BaseEntity, key: str) -> Optional[Tuple[str, str, str]]:  # noqa: C901
    """Get the triples' strings that should be written to the file."""
    data = graph[u][v][key]

    # order is important
    converters = [
        NamedComplexHasComponentConverter,
        PartOfNamedComplexConverter,
        PartOfBiologicalProcess,
        RegulatesActivityConverter,
        MiRNADecreasesExpressionConverter,
        MiRNADirectlyDecreasesExpressionConverter,
        IsAConverter,
        EquivalenceConverter,
        CorrelationConverter,
        AssociationConverter,
        DrugIndicationConverter,
        DrugSideEffectConverter,
        RegulatesAmountConverter,
        IncreasesAmountConverter,
        DecreasesAmountConverter,
    ]

    for converter in converters:
        if converter.predicate(u, v, key, data):
            return converter.convert(u, v, key, data)

    logger.info(f'unhandled: {graph.edge_to_bel(u, v, data)}')
    return None



# -*- coding: utf-8 -*-

"""Tests for the conversion procedure"""

import unittest
from typing import Tuple, Type

from biokeen.convert import get_triple
from biokeen.convert.converters import (
    AssociationConverter, Converter, CorrelationConverter, DecreasesAmountConverter, DrugIndicationConverter,
    DrugSideEffectConverter, IncreasesAmountConverter, MiRNADecreasesExpressionConverter,
    NamedComplexHasComponentConverter, PartOfBiologicalProcess, PartOfNamedComplexConverter, RegulatesActivityConverter,
    RegulatesAmountConverter,
)
from pybel import BELGraph
from pybel.constants import (
    ASSOCIATION, DECREASES, HAS_COMPONENT, INCREASES, NEGATIVE_CORRELATION, OBJECT, PART_OF, POSITIVE_CORRELATION,
    REGULATES, RELATION,
)
from pybel.dsl import (
    Abundance, BaseEntity, BiologicalProcess, MicroRna, NamedComplexAbundance, Pathology, Protein,
    Rna, activity,
)
from pybel.testing.utils import n
from pybel.typing import EdgeData


def _rel(x):
    return {RELATION: x}


def _rela(x, y=None):
    return {RELATION: x, OBJECT: activity(y)}


def _assoc(y):
    return {RELATION: ASSOCIATION, 'association_type': y}


a1 = Abundance('CHEBI', '1')
p1 = Protein('HGNC', '1')
d1 = Pathology('MESH', '1')
b1 = BiologicalProcess('GO', '1')
b2 = BiologicalProcess('GO', '2')
m1 = MicroRna('MIRBASE', '1')
r1 = Rna('HGNC', '1')
r2 = Rna('HGNC', '2')
nca1 = NamedComplexAbundance('FPLX', '1')

converters_true_list = [
    (NamedComplexHasComponentConverter, nca1, p1, _rel(HAS_COMPONENT), ('HGNC:1', 'partOf', 'FPLX:1')),
    (PartOfNamedComplexConverter, p1, nca1, _rel(PART_OF), ('HGNC:1', 'partOf', 'FPLX:1')),
    (PartOfBiologicalProcess, b1, b2, _rel(PART_OF), ('GO:1', 'partOf', 'GO:2')),
    (AssociationConverter, r1, r2, _rel(ASSOCIATION), ('HGNC:1', 'association', 'HGNC:2')),
    (AssociationConverter, r1, r2, _assoc('similarity'), ('HGNC:1', 'similarity', 'HGNC:2')),
    (CorrelationConverter, r1, r2, _rel(POSITIVE_CORRELATION), ('HGNC:1', 'positiveCorrelation', 'HGNC:2')),
    # Found in ADEPTUS
    (CorrelationConverter, d1, r1, _rel(POSITIVE_CORRELATION), ('MESH:1', 'positiveCorrelation', 'HGNC:1')),
    (CorrelationConverter, d1, r1, _rel(NEGATIVE_CORRELATION), ('MESH:1', 'negativeCorrelation', 'HGNC:1')),
    # Found in LINCS (not integrated yet)
    (RegulatesAmountConverter, a1, r1, _rel(REGULATES), ('CHEBI:1', 'regulatesAmountOf', 'HGNC:1')),
    (IncreasesAmountConverter, a1, r1, _rel(INCREASES), ('CHEBI:1', 'increasesAmountOf', 'HGNC:1')),
    (DecreasesAmountConverter, a1, r1, _rel(DECREASES), ('CHEBI:1', 'decreasesAmountOf', 'HGNC:1')),
    # Found in SIDER
    (DrugSideEffectConverter, a1, d1, _rel(INCREASES), ('CHEBI:1', 'increases', 'MESH:1')),
    (DrugIndicationConverter, a1, d1, _rel(DECREASES), ('CHEBI:1', 'decreases', 'MESH:1')),
    # Found in miRTarBase
    (MiRNADecreasesExpressionConverter, m1, r1, _rel(DECREASES), ('MIRBASE:1', 'repressesExpressionOf', 'HGNC:1')),
    # Found in DrugBank
    (RegulatesActivityConverter, a1, p1, _rela(REGULATES), ('CHEBI:1', 'activityDirectlyRegulatesActivityOf', 'HGNC:1'))
    # Found in HSDN
    # 
]

converters_false_list = [
    (NamedComplexHasComponentConverter, nca1, p1, _rel(PART_OF)),
    (PartOfNamedComplexConverter, nca1, p1, _rel(HAS_COMPONENT)),
]


class TestConverters(unittest.TestCase):
    """Tests for the converter classes."""

    def help_test_convert(self,
                          converter: Type[Converter],
                          u: BaseEntity,
                          v: BaseEntity,
                          edge_data: EdgeData,
                          triple: Tuple[str, str, str],
                          ) -> None:
        self.assertTrue(issubclass(converter, Converter), msg=f'Not a Converter: {converter.__name__}')
        key = n()
        self.assertTrue(
            converter.predicate(u, v, key, edge_data),
            msg=f'Predicate failed: {converter.__name__}',
        )
        self.assertEqual(
            triple,
            converter.convert(u, v, key, edge_data),
            msg='Conversion failed: {converter.__name__}',
        )
        graph = BELGraph()
        graph.add_edge(u, v, key=key, **edge_data)
        self.assertEqual(
            triple,
            get_triple(graph, u, v, key),
            msg=f'get_triple failed: {converter.__name__}',
        )

    def test_converters_true(self):
        """Test passing converters."""
        for converter, u, v, edge_data, triple in converters_true_list:
            with self.subTest(msg=f'Converter: {converter.__qualname__}'):
                self.help_test_convert(converter, u, v, edge_data, triple)

    def test_converters_false(self):
        """Test falsification of converters."""
        for converter, u, v, edge_data in converters_false_list:
            with self.subTest():
                self.assertFalse(converter.predicate(u, v, n(), edge_data))



# -*- coding: utf-8 -*-

"""Helper script to query parameters needed in training mode."""

import click

from biokeen.constants import ID_TO_DATABASE_MAPPING


def select_database():
    """Select database."""
    click.secho("Current Step: Please select the database you want to train on:", fg='blue')
    for model, model_id in sorted(ID_TO_DATABASE_MAPPING.items()):
        click.echo(f'{model}: {model_id}')

    ids = list(ID_TO_DATABASE_MAPPING.keys())

    while True:
        user_input = click.prompt('> Please select one of the options: ', type=int)

        if user_input in ID_TO_DATABASE_MAPPING:
            return ID_TO_DATABASE_MAPPING[user_input]

        elif user_input in ID_TO_DATABASE_MAPPING.values():
            return user_input

        click.echo(
            f"Invalid input, please type in a number between 1 and {len(ID_TO_DATABASE_MAPPING)} indicating the "
            f"database id.\nFor example, type {ids[0]} to select {ID_TO_DATABASE_MAPPING[ids[0]]} and press enter\n"
        )



# -*- coding: utf-8 -*-

"""Functions for ensuring data sources."""

import logging
import os
from typing import Iterable, Optional

import numpy as np

from pybel import from_pickle, to_pickle
from .constants import DATA_DIR
from .convert import to_pykeen_file

__all__ = [
    'iterate_source_paths',
    'ensure_hippie',
    'ensure_drugbank',
    'ensure_compath',
]

logger = logging.getLogger(__name__)

hippie_bel_path = os.path.join(DATA_DIR, 'hippie.bel.pickle')
hippie_keen_path = os.path.join(DATA_DIR, 'hippie.keen.tsv')

drugbank_bel_path = os.path.join(DATA_DIR, 'drugbank.bel.pickle')
drugbank_keen_path = os.path.join(DATA_DIR, 'drugbank.keen.tsv')

compath_bel_path = os.path.join(DATA_DIR, 'compath.bel.pickle')
compath_keen_path = os.path.join(DATA_DIR, 'compath.keen.tsv')


def get_full_matrix():
    """Get all built data as a matrix."""
    return np.concatenate([
        np.loadtxt(fname, dtype=str, delimiter='\t')
        for fname in iterate_source_paths()
    ])


def iterate_source_paths() -> Iterable[str]:
    """Iterate over the source paths."""
    for file_name in os.listdir(DATA_DIR):
        if 'keen.tsv' in file_name:
            yield os.path.join(DATA_DIR, file_name)


def ensure_hippie(connection: Optional[str] = None):
    """Ensure the HIPPIE database is built."""
    import bio2bel_hippie

    if os.path.exists(hippie_bel_path):
        logger.info('loading from pickle')
        hippie_graph = from_pickle(hippie_bel_path)
    else:
        logger.info('getting from manager')
        hippie_manager = bio2bel_hippie.Manager(connection=connection)
        if not hippie_manager.is_populated():
            hippie_manager.populate()
        hippie_graph = hippie_manager.to_bel()

        to_pickle(hippie_graph, hippie_bel_path)

    logger.info('outputting')
    to_pykeen_file(hippie_graph, hippie_keen_path)


def ensure_drugbank(connection: Optional[str] = None):
    """Ensure the DrugBank database is built."""
    import bio2bel_drugbank

    if os.path.exists(drugbank_bel_path):
        drugbank_graph = from_pickle(drugbank_bel_path)
    else:
        drugbank_manager = bio2bel_drugbank.Manager(connection=connection)
        if not drugbank_manager.is_populated():
            drugbank_manager.populate()
        drugbank_graph = drugbank_manager.to_bel()
        to_pickle(drugbank_graph, drugbank_bel_path)

    to_pykeen_file(drugbank_graph, drugbank_keen_path)


def ensure_compath():
    """Ensure the ComPath mappings are retrieved."""
    import compath_resources
    graph = compath_resources.get_bel()
    to_pickle(graph, compath_bel_path)
    to_pykeen_file(graph, compath_keen_path)
